// Smooth Moods Machine
// 4/13/12
// Gordon McComb

// Function declarations
void PrintSerial(char *message, boolean lf=true);
void PrintSerial(int message, boolean lf=true);

//if in DEBUG mode:
//  Set up SoftwareSerial to MIDI board, use UART serial for Serial Monitor debugging
//  1 = DEBUG mode, 0 otherwise
#define DEBUG 0

// Object library includes
#include <IRremote.h>
#include <TimerOne.h>
#include <EEPROM.h>
#include <TimedAction.h>
#include "constants.h"

#if DEBUG
#include <SoftwareSerial.h> 
  SoftwareSerial MIDI(255, midiIn);       // Rx, Tx (Rx not used)
#endif

// ---- General setup values ---------------------------------
// Volume settings
byte m_velocity = 127;           // Highest note volume
byte masterVolume = 127;         // Master channel volume

char instrument = 0;

// Chord structure
typedef struct __chord {
  char notes[CHORD_NOTES];
  byte channel;
  byte octave;  
  int attack;
  int sustain;
  int release;
  int quiet;
} Chord;
Chord c_chord = { {60, 64, 67}, 0, 0, 30, 2000, 30, 100 };
Chord f_chord = { {65, 69, 72}, 1, 0, 30, 2000, 30, 100 };
Chord g_chord = { {67, 71, 74}, 2, 0, 30, 2000, 30, 100 };

// Chord randomizing structure
typedef struct __randomize {
  int attackMin;
  int attackMax;  
  int sustainMin;
  int sustainMax;  
  int releaseMin;
  int releaseMax;  
  int quietMin;
  int quietMax;  
} Randomize;
Randomize r_rand = { 10, 60, 1000, 5000, 10, 60, 25, 500 };

byte octaveShift = 3;            // Octave randomizer (shift down from middle)

// ---- IRremote functions -----------------------------------
int RECV_PIN = IRSignalIn;       // IR input pin
IRrecv irrecv(RECV_PIN);
decode_results results;

// ---- Chimes -----------------------------------------------
TimedAction chimeOn = TimedAction(CHIMEON_DELAY, chimeOnFunc);
TimedAction chimeOff = TimedAction(CHIMEOFF_DELAY, chimeOffFunc);
char chime_root = 65;
char chimes[][3] = {
  {26,  0,   m_velocity},
  {27,  0,   m_velocity},
  {32,  0,   m_velocity},
  {112, -24, m_velocity}
};
byte chimeNotes[][3] = {
  {chime_root,    chime_root+4, 0}, 
  {chime_root+8,  chime_root+12, 0}, 
  {chime_root+16, chime_root+20, 0},
  {chime_root+24, chime_root+28, 0},
  {chime_root+32, chime_root+36, 0},    
};
byte chimeSets = (sizeof(chimeNotes) / sizeof(byte)) / (sizeof(chimeNotes[0]) / sizeof(byte));

// Surf
TimedAction surfer = TimedAction(SURF_DELAY, surfFunc);
boolean surfActive = false;

// Reset delay timer
TimedAction resetFlag = TimedAction(RESET_FLAG_DELAY, resetFlagFunc);

// ---- Interrupt vars ---------------------------------------
volatile boolean interruptFlag, standby, irDecode = false;
volatile boolean firstPlay = true;
volatile byte mode = 8;
volatile unsigned long lastInterval = 0;

// ---- Misc variables ---------------------------------------
unsigned long timeOut = millis();
unsigned long timeOutInterval = TIME_OUT_INTERVAL;

// ---- setup ------------------------------------------------
void setup() {
  
  // Set up infrared receiver on pins 5, 6, 7
  pinMode(IRGround, OUTPUT);       // IR power, ground pins
  pinMode(IRPower, OUTPUT);
  digitalWrite(IRGround, LOW);     // IR ground
  digitalWrite(IRPower, HIGH);     // IR power
  irrecv.enableIRIn();             // Start the receiver
  
// Set up serial and MIDI
#if DEBUG
  MIDI.begin(31250);               // Set up serial comm to MIDI
  Serial.begin(115200);
#else
  Serial.begin(31250);
#endif

  resetMidi();                     // Reset MIDI device
  delay(1);
 
  //Seed random generator
  int rand = analogRead(0);
  randomSeed(rand);
  int randArray[] = {254, 904, 163, 932, 197, 6, 77, 638, 317, 738, 82};
  randomSeed((long)rand * random(randArray[random(9)]));
  
  pinMode(13, OUTPUT);             // LED for interruptFlag status
  
  //Get previously stored mode
  mode = EEPROM.read(PROM_MODE);
  if(mode >= 10)                  // Not previously set, or invalid force to chords mode
    mode = 8;
  PrintSerial("PROM_MODE: ", false);
  PrintSerial(mode);
  
  setMode(mode);
  delay(100);
  
  // Set up background timer
  Timer1.initialize(BG_TIMER_INTERVAL);
  Timer1.attachInterrupt(bgTimer);
  
  ctrlMIDI(CHIME, CTRL_BANK, BANK_MELODIC);
 
}

// ---- Main loop --------------------------------------------
void loop() {
  
  // Mute output if shutoff time out has elapsed
  if(timeOutInterval != 0) {
    if(millis() - timeOut > timeOutInterval) {
      standby = true;
    }
  }
  
  // Skip loop if interruptFlag is still active
  while(interruptFlag)
    delayNoBlock(1);
  
  if(!standby) {
    switch(mode) {
      case 1: case 2: case 3: case 4:
        chime();
        break;
      case 5: 
        // pink noise
        pink();
        break;      
      case 6:
        // variable pink noise
        pink_variable();
        break;
      case 7:        // Surf
        surfActive = false;
        surf();
        break;
      case 8:        // Chords
        chords();
        break;
      case 9:        // Thrum
        thrum();
        break;
      case 10:
        // Not used
        break;
    }
  } else {
    delay(20);
  }
}

// Background timer interrupt routine
void bgTimer() {
  resetFlag.check();                // Check interruptFlag timer
  if(!irDecode) {
    // If valid value was received
    if ( irrecv.decode(&results) ) {
      irDecode = true;              // Skip if still processing old IR code
      IRReceiver(results.value);
      irrecv.resume();              // Receive the next value
      irDecode = false;
    }
  }
}

// Reset interrupt flag (delayed from bgTimer)
void resetFlagFunc() {
  interruptFlag = false;
}

// Non-blocking delay; break loop early if interruptFlag is set
void delayNoBlock(unsigned long ms) {
  uint16_t start = (uint16_t)micros();
  while ((ms > 0) && (interruptFlag == false)) {
    if (((uint16_t)micros() - start) >= 1000) {
      ms--;
      start += 1000;
    }
  }
}

// Fetch value from IR remote
void IRReceiver(long value) {
  
  digitalWrite(13, HIGH);                 // Activate pin 13 LED (status check)
    
#if DEBUG
  PrintSerial("Received value: ", false);
  Serial.println(value, HEX);
#endif

  byte button = mode;
  // Skip if value = 0 (error), or if last button was< 750 ms ago
  if((value != 0) && (millis() - lastInterval > 750)) {   
    switch (value) {                      // Match button against Sony codes
      case IR_BUTTON_1:
        button = 1;
        break;
      case IR_BUTTON_2:
        button = 2;
        break;
      case IR_BUTTON_3:
        button = 3;
        break;
      case IR_BUTTON_4:
        button = 4;
        break;
      case IR_BUTTON_5:
        button = 5;
        break;
      case IR_BUTTON_6:
        button = 6;
        break;
     case IR_BUTTON_7:
        button = 7;
        break;
      case IR_BUTTON_8:
        button = 8;
        break;
      case IR_BUTTON_9:
        button = 9;
        break;
      case IR_BUTTON_0:
        // Not used
        break;
      case IR_MUTE:
        //Not used
        break;
      case IR_CHANNEL_DOWN:
        // Not used
        break;
      case IR_CHANNEL_UP:
        // Not used
        break;
      case IR_VOL_UP:
        // Not used
        break;
      case IR_VOL_DOWN:
        // Not used
        break;
      case IR_POWER:
        standby = !standby;
        if(standby) {
          setMode(11);         // Standby
        } else {
          setMode(mode);       // Normal
        }
        resetFlag.reset();
        interruptFlag = true;
        lastInterval = millis();
        digitalWrite(13, LOW);
        return;
        break;
        
      default:
        return;
        break;
        
    }

    // Store mode to EEPROM if changed
    if (button != mode) {
      mode = button;
      EEPROM.write(PROM_MODE, button);
    }
    
    // Reset timers and interruptFlag if new mode is set
    if(button > 0 && button <= 10) {
      resetFlag.reset();
      interruptFlag = true;
      setMode(mode);
    }


    lastInterval = millis();    
  }
  
  digitalWrite(13, LOW);                  // Deactivate pin 13 LED
  
}

// ---- Mode setup -------------------------------------------
void setMode(byte thisMode) {
  cancelAll();
  firstPlay = true;
  if(thisMode != 11) {                   // If button other than power (on/off mute)
    timeOut = millis();
    standby = false;
 }
}

//------------------------------------------------------------
// ---- Pink noise ----

// Constant volume pink noise
void pink() {
  
  if(interruptFlag)
    return;

  // Set up instrument and channel if first time through the loop
  if(firstPlay) {
    UITone();
    instrument = SURF_INSTRUMENT ;
    ctrlMIDI(SURF, CTRL_VOLUME, 127);
    sendMIDI(CTRL_INSTRUMENT|SURF, instrument, 0);
    noteOn(SURF, 60, m_velocity);    
    standby = false;
    firstPlay = false;
  }
   
  while(!interruptFlag) ;
  
  cancelAll();
  
}

// Variable volume pink noise
void pink_variable() {
  
  if(interruptFlag)
    return;

  // Set up instrument and channel if first time through the loop
  if(firstPlay) {
    UITone();
    instrument = SURF_INSTRUMENT ;
    ctrlMIDI(SURF, CTRL_VOLUME, 127);
    sendMIDI(CTRL_INSTRUMENT|SURF, instrument, 0);
    noteOn(SURF, 60, m_velocity);    
    standby = false;
    firstPlay = false;
  }
   
  while(!interruptFlag) {
    
  // Randomized swell-down delay
  for(int i=m_velocity; i>=PINK_MIN; i--) {
    ctrlMIDI(SURF, CTRL_EXPRESS, i); 
    delayNoBlock(random(PINK_SURGE_MIN+20, PINK_SURGE_MAX+20));
    if(interruptFlag) {
      break;
    }
  }
  if(interruptFlag) {
     cancelAll();
     return;
  }
    
  // Randomized sustain delay
  delayNoBlock(random(1000, 5000));
  if(interruptFlag) {    
    cancelAll();
    return;
  }

  // Randomized swell-up delay
  for(int i=PINK_MIN; i<=m_velocity; i++) {       // Attack
    ctrlMIDI(SURF, CTRL_EXPRESS, i);
    delayNoBlock(random(PINK_SURGE_MIN, PINK_SURGE_MAX));
    if(interruptFlag) {
      break;      
    }
  }
  if(interruptFlag) {
     cancelAll();
     return;
  }

 }

  cancelAll(); 
}

//------------------------------------------------------------
// ---- Surf ----

// Main surf loop
void surf() {
  
  if(interruptFlag)
    return;

  if(firstPlay) {
    UITone();
    instrument = SURF_INSTRUMENT;
    ctrlMIDI(SURF, CTRL_VOLUME, 0);
    sendMIDI(CTRL_INSTRUMENT|SURF, instrument, 0);
    standby = false;
    firstPlay = false;
  }
  
  surfer.check();
  delay(2);
}

void surfFunc() {

  // Skip function is surf cycle is already active
  if (!surfActive) {
    surfActive = true;
    byte surfVolume = random(50, 127);
    ctrlMIDI(SURF, CTRL_VOLUME, surfVolume);    // Set top volume level (0 to 127)
    ctrlMIDI(SURF, CTRL_EXPRESS, 0);
    
    noteOn(SURF, 60, m_velocity);               // Turn on "note"
    
    // Attack
    for(int i=0; i<=m_velocity; i++) {
      ctrlMIDI(SURF, CTRL_EXPRESS, i);
      delayNoBlock(random(SURF_SURGE_MIN, SURF_SURGE_MAX));
      if(interruptFlag) {
        cancelAll();
        break;
        return;
      }
    }
    
    // Sustain
    delayNoBlock(random(SURF_SUSTAIN_MIN, SURF_SUSTAIN_MAX));
    if(interruptFlag) {
      cancelAll();
      return;
    }
    
    // Release
    for(int i=m_velocity; i>=0; i--) {
      ctrlMIDI(SURF, CTRL_EXPRESS, i); 
      delayNoBlock(random(SURF_SURGE_MIN, SURF_SURGE_MAX));
      if(interruptFlag) {
        cancelAll();
        break;
        return;
      }
  }

    noteOff(SURF, 60, 0);                       // Turn note off
    surfActive = false;
  }
  
}

//------------------------------------------------------------
// ---- Chimes ----

// Main chimes loop
void chime() {

  if (interruptFlag)
    return;

  if(firstPlay) {
    UITone();
    instrument = chimes[mode-1][0];   // Select chime instrument from array
    ctrlMIDI(CHIME, CTRL_ALL_OFF, 0);
    sendMIDI(CTRL_INSTRUMENT|CHIME, instrument, 0);
    ctrlMIDI(CHIME, CTRL_VOLUME, 127);
    standby = false;
    firstPlay = false;
  }

  chimeOn.check();                    // Call either/both chime functions if timer has elapsed
  chimeOff.check();
  delay(20);

}

// Turn chime note on
void chimeOnFunc() {
  byte chimeDelay = random(0, CHIME_RANDOMIZER);
  if(chimeDelay<chimeSets) {
    byte thisNote = random(0, chimeSets);
    byte velo = chimes[thisNote][2];
    noteOn(CHIME, chimeNotes[thisNote][0]+chimes[mode-1][1], velo);
    delayNoBlock(CHIME_STEP);
    if(interruptFlag) {
      return;
    }
    noteOn(CHIME, chimeNotes[thisNote][1]+chimes[mode-1][1], velo);
    if(random(0, 2) == 1) {
      delayNoBlock(CHIME_STEP*2);
      if(interruptFlag) {
        return;
      }
      noteOn(CHIME, chimeNotes[thisNote][1]+chimes[mode-1][1]+2, velo);
    }
    chimeNotes[thisNote][2]=1;        // Note group sounding flag on
  }
}

// Turn chime note off (at random, if note is on)
void chimeOffFunc() {
  byte thisNote = random(0, (chimeSets * 9));
  if(chimeNotes[thisNote][2] == 1) {  // If note flag
    noteOff(CHIME, chimeNotes[thisNote][1]+chimes[mode-1][1], 0);
    noteOff(CHIME, chimeNotes[thisNote][0]+chimes[mode-1][1], 0);
    noteOff(CHIME, chimeNotes[thisNote][1]+chimes[mode-1][1]+2, 0);
    chimeNotes[thisNote][2]=0;        // Reset sounding flag
  }
}

//------------------------------------------------------------
// ---- Major chords ----

// Main chord loop
void chords() {
  
  if(interruptFlag)
    return;

  if(firstPlay) {
    UITone();
    instrument = CHORD_INSTRUMENT;
    ctrlMIDI(CHIME, CTRL_VOLUME, 0);    
    for (int channel=0; channel<NUM_CHANNELS; channel++) {
      ctrlMIDI(channel, CTRL_VOLUME, 0);
      sendMIDI(CTRL_INSTRUMENT|channel, instrument, 0);
    }
    standby = false;
    firstPlay = false;
  }
    
  // Get first chord to play
  Chord thisChord = getChord();
  chordOn(&thisChord);
  
  // Get the next chord to play
  Chord nextChord = getChord();
  chordOn(&nextChord);
  
  // Set up volume for first chord
  ctrlMIDI(thisChord.channel, CTRL_VOLUME, m_velocity);
  
  // First chord swell up for attack
  for(int i = 0; i<=127; i++) {
    ctrlMIDI(c_chord.channel, CTRL_EXPRESS, i); 
    ctrlMIDI(f_chord.channel, CTRL_EXPRESS, i);
    ctrlMIDI(g_chord.channel, CTRL_EXPRESS, i);
    delayNoBlock(thisChord.attack);
    if(interruptFlag) {
      cancelAll();
      break;
      return;
    }
  }

  // First chod delay at top of swell
  delayNoBlock(thisChord.sustain/2);
  if(interruptFlag) {
    cancelAll();
    return;
  }  
  
  // Sustain
  // Cross-fade to second chord at top of swell; skip if same chord
  if (thisChord.channel != nextChord.channel) {
    for ( int i = 0; i<=m_velocity; i++ ) {
      ctrlMIDI(nextChord.channel, CTRL_VOLUME, i);
      ctrlMIDI(thisChord.channel, CTRL_VOLUME, (m_velocity-i)+(m_velocity-127));
      delayNoBlock(CROSS_FADE);
      if(interruptFlag)
        break;
    }
    delayNoBlock(thisChord.sustain/2);
    if(interruptFlag) {
      cancelAll();
      return;
    } else {
      ctrlMIDI(thisChord.channel, CTRL_VOLUME, 0);
      ctrlMIDI(nextChord.channel, CTRL_VOLUME, m_velocity);
    }
  }
  
  // Second chord delay at top of swell
  delayNoBlock(nextChord.sustain/2);
  if(interruptFlag) {
    cancelAll();
    return;
  }
  
  // Second chord swell down for release
  for(int i = 127; i>=0; i--) {
    ctrlMIDI(c_chord.channel, CTRL_EXPRESS, i); 
    ctrlMIDI(f_chord.channel, CTRL_EXPRESS, i);
    ctrlMIDI(g_chord.channel, CTRL_EXPRESS, i);
    delayNoBlock(nextChord.release);
    if(interruptFlag) {
      cancelAll();
      break;
      return;
    }
  }

  // Turn off all notes on every chord channel; reset volume
  notesOff();
  resetVolume();
  
  // Wait quiet period after all sound has stopped
  if(!interruptFlag) {
    delayNoBlock(nextChord.quiet);
    if(interruptFlag) {
      cancelAll();
      return;
    }
  }
  
}

// Activate the random chord, set randomizing values 
//  Note chord is passed in by reference
void chordOn(struct __chord* x_chord) {
  for(int i=0; i<3; i++) {
    x_chord->octave = random(octaveShift);
    x_chord->attack = random(r_rand.attackMin, r_rand.attackMax);
    x_chord->sustain = random(r_rand.sustainMin, r_rand.sustainMax);
    x_chord->release = random(r_rand.releaseMin, r_rand.releaseMax);    
    x_chord->quiet = random(r_rand.quietMin, r_rand.quietMax);     
    noteOn(x_chord->channel, x_chord->notes[i]-(12*x_chord->octave), m_velocity); 
  }
}

// Get a random chord, from 0 to 2 
struct __chord getChord() {
  int rand = random(3);
  Chord x_chord;
  switch(rand) {
    case 0: 
      x_chord = c_chord;
      break;
    case 1: 
       x_chord = f_chord;
      break;
    case 2: 
      x_chord = g_chord;
      break;
  }
  return (x_chord);
}

//------------------------------------------------------------
// ---- Thrum ----

// Main thrum loop
void thrum() {
  
  if(interruptFlag)
    return;

  // Set up instrument and channel if first time through the loop
  if(firstPlay) {
    UITone();
    instrument = THRUM_INSTRUMENT;
    ctrlMIDI(THRUM, CTRL_VOLUME, 0);
    sendMIDI(CTRL_INSTRUMENT|THRUM, instrument, 0);
    standby = false;
    firstPlay = false;
  }
  
  // Get random pitch from low and high limits
  byte pitch = random(THRUM_LOW, THRUM_HIGH);
  ctrlMIDI(THRUM, CTRL_VOLUME, m_velocity);  
  noteOn(THRUM, pitch, m_velocity);
  noteOn(THRUM, pitch+4, m_velocity);
  noteOn(THRUM, pitch+7, m_velocity);
  
  // Randomized swell-up delay
  for(int i=0; i<=m_velocity; i++) {       // Attack
    ctrlMIDI(THRUM, CTRL_EXPRESS, i);
    delayNoBlock(random(1, 10));
    if(interruptFlag) {
      break;      
    }
  }
  if(interruptFlag) {
     cancelAll();
     return;
  }
  
  // Randomized sustain delay
  delayNoBlock(random(750, 4500));
  if(interruptFlag) {    
    cancelAll();
    return;
  }
  
  // Randomized swell-down delay
  for(int i=m_velocity; i>=0; i--) {
    ctrlMIDI(THRUM, CTRL_EXPRESS, i); 
    delayNoBlock(random(8, 200));
    if(interruptFlag) {
      break;
    }
  }
  if(interruptFlag) {
     cancelAll();
     return;
  }

  // Randomized quiet delay between
  delayNoBlock(random(500, 2000));
  if(interruptFlag) {
    cancelAll();
    return;
  }
  
  ctrlMIDI(THRUM, CTRL_ALL_OFF, 0);
  
}

//----Utility functions---------------------------------------

// Reset volume all channels
void resetVolume() {
  for (int channel=0; channel<NUM_CHANNELS; channel++) {
    ctrlMIDI(channel, CTRL_VOLUME, 0);
  }
}

// Reset notes all channels
void notesOff() {
  for(int channel=0; channel<NUM_CHANNELS; channel++) {
    ctrlMIDI(channel, CTRL_ALL_OFF, 0);
  }
}

// Reset both volume and notes all channels
void resetChannels() {
  for (int channel=0; channel<NUM_CHANNELS; channel++) {
    ctrlMIDI(channel, CTRL_ALL_OFF, 0);
    ctrlMIDI(channel, CTRL_VOLUME, 0);
  }
}

// Reset expression all chord channels
void resetExpression() {
  ctrlMIDI(c_chord.channel, CTRL_EXPRESS, m_velocity); 
  ctrlMIDI(f_chord.channel, CTRL_EXPRESS, m_velocity);
  ctrlMIDI(g_chord.channel, CTRL_EXPRESS, m_velocity); 
}

// Reset all channel volume and expression, turn notes off
void cancelAll() {
  for(int i=0; i<=NUM_CHANNELS; i++) {
    ctrlMIDI(i, CTRL_ALL_OFF, 0);
    ctrlMIDI(i, CTRL_VOLUME, 0);
    ctrlMIDI(i, CTRL_EXPRESS, m_velocity); 
  }
}

// Sound distinctive tone to indicate selected mode
void UITone() {
  sendMIDI(CTRL_INSTRUMENT|UI, 0, 0);
  ctrlMIDI(UI, CTRL_VOLUME, 127);

  // Chime mode (1 to 5) 
  if(mode>0 && mode <=4) {
    noteOn(UI, 60, m_velocity);
    delay(50);
    noteOn(UI, 64, m_velocity);    
  }
  
  // Surf, chord, thrum, or pink noise mode
  switch(mode) {
    case 5:
      noteOn(UI, 48, m_velocity);
      delay(100);
      noteOn(UI, 48, m_velocity);
      break;    
    case 6:
      noteOn(UI, 53, m_velocity);
      delay(50);
      noteOn(UI, 53, m_velocity);
      break;
    case 7:
      noteOn(UI, 40, m_velocity);
      delay(100);
      noteOn(UI, 47, m_velocity);
      break;
    case 8:
      noteOn(UI, 50, m_velocity);
      delay(100);
      noteOn(UI, 57, m_velocity);
      break;
    case 9:
      noteOn(UI, 60, m_velocity);
      delay(100);      
      noteOn(UI, 67, m_velocity);
      break;
  }
  delay(100);
}

//------------------------------------------------------------
// Debug and serial print routines

// Print variables for chords
void printVals(struct __chord* x_chord) {  
  PrintSerial(x_chord->notes[0]);
  PrintSerial(x_chord->channel);
  PrintSerial(x_chord->attack);  
  PrintSerial(x_chord->sustain);  
  PrintSerial(x_chord->release);
  PrintSerial(x_chord->quiet);  
  PrintSerial("");
}

// Print to Serial Monitor
void PrintSerial(char *message, boolean lf) {
#if DEBUG
  if(lf)
    Serial.println (message);
  else
    Serial.print(message);
#endif
}
void PrintSerial(int message, boolean lf) {
#if DEBUG
  if(lf)
    Serial.println (message, DEC);
  else
    Serial.print(message, DEC);
#endif
}

//------------------------------------------------------------
// ---- Standard MIDI functions ----

// Turn note on (press key)
void noteOn(byte channel, byte note, byte attackVelo) {
  sendMIDI( (NOTE_ON | channel), note, attackVelo);
}

// Turn note off (release key)
void noteOff(byte channel, byte note, byte releaseVelo) {
  sendMIDI( (NOTE_OFF | channel), note, releaseVelo);
}

// Set controller value with channel
void ctrlMIDI(byte channel, byte data1, byte data2) {
  sendMIDI( (CONTROLLER | channel), data1, data2);
}

// Send bytes to MIDI device
void sendMIDI(byte cmd, byte data1, byte data2) {
  if(irDecode)   // Skip if IR decode still in progress
    return;

#if DEBUG
  MIDI.write(cmd);
  MIDI.write(data1);
  // Pass only if valid 2nd byte
  if( (cmd & 0xF0) <= 0xB0 || (cmd & 0xF0) == 0xE0 )
    MIDI.write(data2);
#else
  Serial.write(cmd);
  Serial.write(data1);
  if( (cmd & 0xF0) <= 0xB0 || (cmd & 0xF0) == 0xE0 )
    Serial.write(data2);
#endif
}

// Reset MIDI device
void resetMidi() {
  pinMode(resetPin, OUTPUT);
  digitalWrite(resetPin, LOW);
  delay(100);
  digitalWrite(resetPin, HIGH);
  delay(100);
}

